const TRACK_ATTRIBUTE = 'data-track'
const PROPERTIES_ATTRIBUTE = 'data-track-properties'
const OPTIONS_ATTRIBUTE = 'data-track-options'

const eventMap = []

const attributeToJson = ({attributeKey, el}) => {
  const dataString = el.getAttribute(attributeKey)
  if (!dataString) return {}
  // our monolith returns " as &quot;
  const objString = dataString.replace(/&quot;/g, '"')
  return JSON.parse(objString)
}

const trackingCallbackFactory = (el, domain) =>
  function() {
    domain.get('track_tracking_use_case').execute({
      event: el.getAttribute(TRACK_ATTRIBUTE),
      properties: attributeToJson({attributeKey: PROPERTIES_ATTRIBUTE, el}),
      options: attributeToJson({attributeKey: OPTIONS_ATTRIBUTE, el})
    })
  }

const listenToClicksToTrack = function(domain) {
  const elements = document.querySelectorAll(`[${TRACK_ATTRIBUTE}]`)

  Array.from(elements).forEach((element, idx) => {
    const callback = trackingCallbackFactory(element, domain)
    eventMap.push({element, callback})
    element.addEventListener('click', callback)
  })
}

const removeListeners = function() {
  eventMap.forEach(({element, callback}) => {
    element.removeEventListener('click', callback)
  })
}

document.addEventListener('segment:reload', function() {
  removeListeners()
  listenToClicksToTrack()
})

export default listenToClicksToTrack
